/*
 * Copyright (c) 2012, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#include "CzStack.h"


/* スタックへの追加 */
BOOL CzStack_push_uint32(CZSTACK *stack, UINT32 data)
{
	return CzMemory_add_datas(stack, (char*)&data, sizeof(UINT32));
}

/* スタックからの取得 */
BOOL CzStack_pop_uint32(CZSTACK *stack, UINT32 *retdata)
{
	int i;

	if(stack->size == 0) {
		return FALSE;
	}
	stack->size -= sizeof(UINT32);

	*retdata = 0;
	for(i = 0; i < sizeof(UINT32); i++) {
		*retdata <<= 8;
#ifdef WORDS_BIGENDIAN
		*retdata |= (stack->data[stack->size + i] & 0xff);
#else
		*retdata |= (stack->data[stack->size + sizeof(UINT32) - i - 1]
			& 0xff);
#endif
	}
	return TRUE;
}

/* スタックからオフセット指定して設定 */
BOOL CzStack_set_uint32(CZSTACK *stack, int offset, UINT32 data)
{
	int i, p;

	p = stack->size + sizeof(UINT32) * offset;
	if(p < 0) {
		return FALSE;
	}
	if(p + sizeof(UINT32) > stack->size) {
		return FALSE;
	}

	for(i = 0; i < sizeof(UINT32); i++) {
#ifdef WORDS_BIGENDIAN
		stack->data[p + sizeof(UINT32) - i - 1] = data & 0xff;
#else
		stack->data[p + i] = data & 0xff;
#endif
		data >>= 8;
	}
	return TRUE;
}

/* スタックからオフセット指定しての取得 */
BOOL CzStack_get_uint32(CZSTACK *stack, int offset, UINT32 *retdata)
{
	int i, p;

	p = stack->size + sizeof(UINT32) * offset;
	if(p < 0) {
		return FALSE;
	}
	if(p + sizeof(UINT32) > stack->size) {
		return FALSE;
	}

	*retdata = 0;
	for(i = 0; i < sizeof(UINT32); i++) {
		*retdata <<= 8;
#ifdef WORDS_BIGENDIAN
		*retdata |= (stack->data[p + i] & 0xff);
#else
		*retdata |= (stack->data[p + sizeof(UINT32) - i - 1] & 0xff);
#endif
	}
	return TRUE;
}
