/*
 * Copyright (c) 2012-2013, Masanori Miyamoto.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1.Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * 2.Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */ 
#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <string.h>
#include <ctype.h>
#include "lex_analyzer.h"
#include "parser.h"
#include "CzFStream.h"
#include "CzarBasic.h"
#include "CzarBasic_vm.h"

static VM_DATA st_vmdata;

#ifdef PRINT_STRINGPOOL
static void print_stringpool();
#endif

int main(int argc, char *argv[])
{
	CZFSTREAM stream;
	LEX_DATA lex_data;
	FILE *fp;
	size_t i, len;

	setlocale(LC_ALL, "");

	if(argc != 2) {
		fprintf(stderr, "CzarBasic Version 0.3.0\n");
		fprintf(stderr, "Copyright (C) 2012-2013 by Masanori Miyamoto.\n");
		fprintf(stderr, "\n");
		fprintf(stderr, "usage:%s filename\n", argv[0]);
		exit(1);
	}
#ifdef LEX_TEST
	lex_test(argc, argv);
#endif
	/* ファイルオープン */
	fp = fopen(argv[1], "r");
	if(!fp) {
		fprintf(stderr, "File open error\n");
		exit(1);
	}
	CzFStream_init(&stream, fp);
	lex_data_init(&lex_data, (CZSTREAM*)&stream);
	vm_data_init(&st_vmdata);

	/* 解析 */
	if(parser(&st_vmdata, &lex_data) == PARSE_RESULT_FALSE) {
		stream.super.close((CZSTREAM*)&stream);
#ifdef PRINT_STRINGPOOL
		print_stringpool();
#endif
#ifdef PRINT_LABEL_TABLE
		print_label_table(&st_vmdata);
#endif
#ifdef PRINT_LINE_NUM_TABLE
		print_line_num_table(&st_vmdata);
#endif
#ifdef PRINT_DISASM
		vm_disasm(&st_vmdata);
#endif
		lex_data_delete(&lex_data);
		ErrorMessage(lex_data.err_code, lex_data.err_line_num);
		exit(1);
	}
	stream.super.close((CZSTREAM*)&stream);
	lex_data_delete(&lex_data);

#ifdef PRINT_STRINGPOOL
	print_stringpool();
#endif
#ifdef PRINT_LABEL_TABLE
	print_label_table(&st_vmdata);
#endif
#ifdef PRINT_LINE_NUM_TABLE
	print_line_num_table(&st_vmdata);
#endif
#ifdef PRINT_DISASM
	vm_disasm(&st_vmdata);
#endif
	/* 実行 */
	if(!vm_run(&st_vmdata)) {
		ErrorMessage(st_vmdata.err_code, search_editor_line_num(&st_vmdata, st_vmdata.pc - 1));
		exit(1);
	}
	vm_data_delete(&st_vmdata);

	return 0;
}

#ifdef PRINT_STRINGPOOL
static void print_stringpool()
{
	size_t i, j;

	CZSTRING *string_pool = (CZMEMORY*)st_vmdata.string_pool.data;
	for(i = 0; i < st_vmdata.string_pool.size / sizeof(CZSTRING); i++) {
		printf("string_pool:%d:", i);
		for(j = 0; j < string_pool[i].size; j++) {
			printf("%02X ", string_pool[i].data[j] & 0xff);
		}
		printf("\n");
	}
}
#endif
